<?php

namespace Drupal\khalti\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\user\UserInterface;

/**
 * Defines the KhaltiTransaction entity.
 *
 * @ingroup khalti
 *
 * @ContentEntityType(
 *   id = "khalti_transaction",
 *   label = @Translation("KhaltiTransaction"),
 *   handlers = {
 *     "view_builder" = "Drupal\khalti\KhaltiTransactionViewBuilder",
 *     "list_builder" = "Drupal\khalti\KhaltiTransactionListBuilder",
 *     "views_data" = "Drupal\khalti\Entity\KhaltiTransactionViewsData",
 *     "translation" = "Drupal\khalti\KhaltiTransactionTranslationHandler",
 *
 *     "form" = {
 *       "default" = "Drupal\khalti\Form\KhaltiTransactionForm",
 *       "add" = "Drupal\khalti\Form\KhaltiTransactionForm",
 *       "edit" = "Drupal\khalti\Form\KhaltiTransactionForm",
 *       "delete" = "Drupal\khalti\Form\KhaltiTransactionDeleteForm",
 *       "delete-multiple-confirm" = "Drupal\khalti\Form\ConfirmDeleteMultipleKhaltiTransaction",
 *     },
 *     "access" = "Drupal\khalti\KhaltiTransactionAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\khalti\KhaltiTransactionHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "khalti_transaction",
 *   data_table = "khalti_transaction_field_data",
 *   translatable = FALSE,
 *   admin_permission = "administer khalti_transaction entities",
 *   fieldable = TRUE,
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "transaction_id",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "canonical" = "/admin/commerce/khalti-transaction/{khalti_transaction}",
 *     "add-form" = "/admin/commerce/khalti-transaction/add",
 *     "edit-form" = "/admin/commerce/khalti-transaction/{khalti_transaction}/edit",
 *     "delete-form" = "/admin/commerce/khalti-transaction/{khalti_transaction}/delete",
 *   },
 *   field_ui_base_route = "khalti_transaction.settings"
 * )
 */
class KhaltiTransaction extends ContentEntityBase implements KhaltiTransactionInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    return $this->get('transaction_id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setTitle($title) {
    $this->set('transaction_id', $title);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($created) {
    $this->set('created', $created);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOrder() {
    return $this->get('order_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setChangedTime($published) {
    // $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getChangedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerName() {
    $author = $this->get('uid')->entity->name->value;
    $author = ($author != '') ? $author : 'Anonymous';
    return $author;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Khalti Transaction ID'))
      ->setDescription(t('The ID of the khalti_transaction.'))
      ->setReadOnly(TRUE)
      ->setSetting('unsigned', TRUE);

    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('The khalti_transaction UUID.'))
      ->setReadOnly(TRUE);

    $fields['transaction_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Transaction id'))
      ->setDescription(t('Id of the transaction.'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 255,
      ])
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 4,
      ])
      ->setSetting('unsigned', TRUE)
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $status_options = [
      'completed' => t('Completed'),
      'pending' => t('Pending'),
      'canceled' => t('Canceled'),
      'expired' => t('Expired'),
      'user canceled' => t('User canceled'),
      'initiated' => t('Initiated'),
    ];

    $fields['status'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Transaction status'))
      ->setDescription(t('The status of the transaction.'))
      ->setSettings([
        'allowed_values' => $status_options,
        'default_value' => 'completed',
      ])
      ->setRequired(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $mode_options = [
      'test' => t('Test'),
      'live' => t('Live'),
    ];

    $fields['mode'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Transaction mode'))
      ->setDescription(t('The mode of the transaction.'))
      ->setSettings([
        'allowed_values' => $mode_options,
        'default_value' => 'test',
      ])
      ->setRequired(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 6,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 6,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['order_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Order'))
      ->setReadOnly(TRUE)
      ->setDescription(t('The order for the transaction.'))
      ->setSetting('target_type', 'commerce_order')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 7,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 7,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['pidx'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Pidx'))
      ->setDescription(t('The pidx for the transaction.'))
      ->setSetting('unsigned', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Transaction author'))
      ->setReadOnly(TRUE)
      ->setDescription(t('The user for the transaction.'))
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback('Drupal\khalti\Entity\KhaltiTransaction::getCurrentUserId');

    $fields['langcode'] = BaseFieldDefinition::create('language')
      ->setLabel(t('Language code'))
      ->setDescription(t('The khalti_transaction language code.'));

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('When the khalti_transaction was created, as a Unix timestamp.'));

    $fields['updated'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Last updated'))
      ->setDescription(t('The time when the transaction was last updated.'))
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    // Use the current timestamp.
    $this->set('updated', REQUEST_TIME);
    parent::preSave($storage);
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    parent::postDelete($storage, $entities);

    // Codes here.
  }

  /**
   * Default value callback for 'uid' base field definition.
   *
   * @see ::baseFieldDefinitions()
   *
   * @return array
   *   An array of default values.
   */
  public static function getCurrentUserId() {
    return [\Drupal::currentUser()->id()];
  }

}
