<?php

namespace Drupal\khalti;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\Client;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Khalti Payment Class.
 */
class Khalti {

  private string $secret_key;
  protected string $return_url;
  protected string $website_url;
  protected string $amount;
  protected string $purchase_order_id;
  protected string $purchase_order_name;
  protected ?array $customer_info;
  protected ?array $amount_breakdown;
  protected ?array $product_details;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $config_factory;

  /**
   * The Guzzle HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $http_client;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * Constructs a Khalti object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \GuzzleHttp\Client $http_client
   *   The Guzzle HTTP client service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, Client $http_client, MessengerInterface $messenger) {
    $this->config_factory = $config_factory;
    $this->http_client = $http_client;
    $this->messenger = $messenger;
  }

  /**
   * Set the Khalti secret key.
   *
   * @param string $secret_key
   *   The secret key.
   */
  public function setSecretKey(string $secret_key) {
    $this->secret_key = $secret_key;
  }

  /**
   * Configure Khalti settings.
   *
   * @param string $return_url
   * @param string $website_url
   * @param string $amount
   * @param string $purchase_order_id
   * @param string $purchase_order_name
   * @param array|null $customer_info
   * @param array|null $amount_breakdown
   * @param array $product_details
   */
  public function config(
    string $return_url,
    string $website_url,
    string $amount,
    string $purchase_order_id,
    string $purchase_order_name,
    ?array $customer_info = NULL,
    ?array $amount_breakdown = NULL,
    ?array $product_details = [],
  ) {
    $this->return_url = $return_url;
    $this->website_url = $website_url;
    $this->amount = $amount;
    $this->purchase_order_id = $purchase_order_id;
    $this->purchase_order_name = $purchase_order_name;
    $this->customer_info = $customer_info;
    $this->amount_breakdown = $amount_breakdown;
    $this->product_details = $product_details;
  }

  /**
   * Initialize the payment and redirect the user to Khalti.
   *
   * @param bool $production
   *   Whether to use the production URL or the sandbox URL.
   */
  public function init(bool $production = FALSE) {
    $url = $production ? 'https://khalti.com/api/v2/epayment/initiate/' : 'https://a.khalti.com/api/v2/epayment/initiate/';

    $postData = [
      "return_url" => $this->return_url,
      "website_url" => $this->website_url,
      "amount" => $this->amount * 100,
      "purchase_order_id" => $this->purchase_order_id,
      "purchase_order_name" => $this->purchase_order_name,
    ];

    if ($this->customer_info) {
      $postData["customer_info"] = $this->customer_info;
    }

    if ($this->amount_breakdown) {
      $postData["amount_breakdown"] = $this->amount_breakdown;
    }

    if ($this->product_details) {
      $postData["product_details"] = $this->product_details;
    }

    try {
      // Make the API request using Guzzle.
      $response = $this->http_client->request('POST', $url, [
        'json' => $postData,
        'headers' => [
          'Authorization' => 'key ' . $this->secret_key,
          'Content-Type' => 'application/json',
        ],
      ]);

      $data = json_decode($response->getBody()->getContents(), TRUE);

      // If the payment URL is returned, redirect to it.
      if (isset($data['payment_url'])) {
        $redirectUrl = $data['payment_url'];
        $response = new RedirectResponse($redirectUrl);
        $response->send();
        exit;
      }
      else {
        // If no payment URL, display an error.
        $this->messenger->addError('Error: Invalid response from Khalti API.');
      }
    }
    catch (\Exception $e) {
      // Catch exceptions and display the error message.
      $this->messenger->addError('Error: ' . $e->getMessage());
    }
  }

  /**
   * Decode the query parameters from the URL.
   *
   * @return array
   *   The decoded data.
   */
  public function decode(): array {
    $data = [
      'pidx' => \Drupal::request()->query->get('pidx', ''),
      'txnId' => \Drupal::request()->query->get('txnId', ''),
      'amount' => \Drupal::request()->query->get('amount', ''),
      'total_amount' => \Drupal::request()->query->get('total_amount', ''),
      'status' => \Drupal::request()->query->get('status', ''),
      'mobile' => \Drupal::request()->query->get('mobile', ''),
      'tidx' => \Drupal::request()->query->get('tidx', ''),
      'purchase_order_id' => \Drupal::request()->query->get('purchase_order_id', ''),
      'purchase_order_name' => \Drupal::request()->query->get('purchase_order_name', ''),
      'transaction_id' => \Drupal::request()->query->get('transaction_id', ''),
    ];
    return $data;
  }

  /**
   * Validate the transaction with Khalti.
   *
   * @param string $transaction_code
   *   The transaction code to validate.
   * @param bool $production
   *   Whether to use the production URL or the sandbox URL.
   *
   * @return mixed
   *   The response from Khalti API.
   */
  public function validate(string $transaction_code, bool $production = FALSE) {
    // Define the URL based on the environment (production or sandbox).
    $url = $production ? 'https://khalti.com/api/v2/epayment/lookup/' : 'https://a.khalti.com/api/v2/epayment/lookup/';

    try {
      // Make the POST request using the Guzzle client.
      $response = $this->http_client->request('POST', $url, [
      // The body of the request.
        'json' => ['pidx' => $transaction_code],
        'headers' => [
          'Authorization' => 'key ' . $this->secret_key,
          'Content-Type' => 'application/json',
        ],
      ]);

      // Return the response body content.
      return $response->getBody()->getContents();
    }
    catch (\Exception $e) {
      // Log the error and show a message.
      \Drupal::messenger()->addError('Error: ' . $e->getMessage());
      // Optionally, you can use `dd()` for debugging.
      // dd($e->getMessage()); // Debugging line.
    }
  }

}
